package anywheresoftware.agraham.b4j.dotnetbridge;

import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

import com.sun.jna.Native;



/**
 *The DotNetBridge object allows access to the methods of a C# MSScript.dll library that has been processed by
 *the UnmanagedExports NuGGet written by Robert Giesecke to make it callable from a B4J Java library via JNA.
 *https://www.nuget.org/packages/UnmanagedExports
 *
 *The C# library exposes the Microsoft ActiveX Script Control which can execute JavaScript and VBScript.
 *Here it is initialized for VBScript.
 *
 *There is very little documentation for the MSScript Control, I obtained that in the xml comments from here.
 *http://msdn.microsoft.com/en-us/library/aa227633(v=vs.60).aspx.
 */
//@DependsOn(values = { "jna-platform-4.0.0", "jna-4.0.0"})
@DependsOn(values = { "jna-4.0.0"})
@Author("Andrew Graham")
@Version(1.0f)
@ShortName("DotNetBridge")
public class DotNetBridge {
	
	// https://sites.google.com/site/robertgiesecke/Home/uploads/unmanagedexports
	
	// 1.0 	initial release
	private static final double version = 1.0;
	
	/**
	 *Returns the version number of the library.
	 */
	public double getVersion() {
		return version;
	}

	/**
	 *This library is a demonstration of how to interface to a C# or VB.NET managed library.
	 *You will need to modify and recompile it to match the managed library you want to use.
	 *There is more information on this in the document UnmanagedExports.rtf in the library archive. 
	 *
	 *This particular library shows that it is possible for a B4J Java library to access a Windows COM or ActiveX object
	 *using COM Interop via a .NET Framework library. In this case the COM object is the MS Script Control.
	 *
	 *This library uses the JNA libraries to call native code methods in a .NET library that have been exported by UnmanagedExports
	 *written by Robert Giesecke.
	 *
	 *The JNA libraries are  licensed under the LGPL, version 2.1 or later, and (from version 4.0 onward)
	 *also the Apache Software License, version 2.0
	 *
	 *This library is therefore also licensed under those licences.
	 *
	 *UnmanagedExports is licensed by Robert Giesecke under the MIT License but  as the managed dll modified by its' use
	 *contains none of the UnmanagedExports code I believe this license applies only to the actual use of UnmanagedExports.
	 *http://opensource.org/licenses/mit-license.php
	 */
	public void LIBRARY_DOC() {
	}
	
	/**
	 *The .NET library looks like an Interface	that extends the JNA Interface com.sun.jna.Library.
	 *The Interface methods map onto the exported .NET library methods and their signatures.
	 */
	public interface MSScript extends com.sun.jna.Library  {			
		// exported method names and signatures			
		public String LibraryName(); 
		public double LibraryVersion();		
		public void About();
		
        public void SetAllowUI(Boolean value);
        public void SetScriptTimeout(int value);
        public void SetUseSafeSubset(boolean value);
       
        public void Reset();
		public void AddCode(String scriptcode);         
        public String Eval(String expression);
        public void ExecuteStatement(String statement);
        public String Run(String startsub, String[] subparams);
        public String ResultType();
       
        public void ClearError();
        public boolean Errored();
        public int ErrorColumn();
        public String ErrorDescription();
        public int ErrorLine();
        public int ErrorNumber();
        public String ErrorSource();
        public String ErrorText();        
        public boolean TimedOut(); 
        
        public boolean ExceptionFlag(); 
        public String ExceptionMsg();
        

	       
    }
	
	MSScript MSmethods; // holds the initialised library Interface
	
	/**
	 *Initialise an instance of our .NET library Interface so we can call its' methods. 
	 */
	public void Initialize() {
		// Load the .NET library which should be in the same folder as the project jar.
		// There is no point in passing the library as a parameter as this entire library  is dll specific
		MSmethods = (MSScript)Native.loadLibrary("MSScript", MSScript.class);
	}
	
	
	
	// Expose the .NET library methods as this library methods and properties

	
	/**
	 *Cause the Script Control to display its' About box. 
	 */
	public void About() {
		MSmethods.About();
	}

	
	/**
	 *Returns the name of the .NET MSScript library.
	 */
	public String DotNetLibraryName() {
		return MSmethods.LibraryName();
	}
	
	/**
	 *Returns the version number of the .NET MSScript library.
	 */
	public double DotNetLibraryVersion() {
		return MSmethods.LibraryVersion();
	}	
	
	
    /**
     *When set to True, means that the script program can display user interface (UI) elements such as a MsgBox or InputBox.
     */
    public void SetAllowUI(Boolean value) {
    	MSmethods.SetAllowUI(value);
    }
    
    /**
     *Sets the maximum amount of time in mS that a script can run before a warning message is displayed (if the AllowUI property is True).
     *-1 = no timeouts will occur.
     * 0 = the ScriptControl will monitor the execution of the script and will trigger the Timeout event if it determines that the script is hung.
     */
    public void SetScriptTimeout(int value) {
    	MSmethods.SetScriptTimeout(value);
    }
    
    /**
     *When set True prevents access to selected objects and procedures that can compromise an applications' security.
     */
    public void SetUseSafeSubset(boolean value) {
    	MSmethods.SetUseSafeSubset(value);
    }
    
    
    /**
     *Reinitializes the scripting engine. 
     */
    public void Reset() {
    	MSmethods.Reset();    	
    }
   
	/**
	 *Add the provided code to the script of the Script Control.
	 */
	public void AddCode(String scriptcode) {
    	MSmethods.AddCode(scriptcode);
	}
	
    /**
     *Evaluate an expression and return the result.
     */
    public String Eval(String expression) {
    	return MSmethods.Eval(expression);
    }
    
    /**
     *Execute the single provided VBScript statement.
     */
    public void ExecuteStatement(String statement) {
    	MSmethods.ExecuteStatement(statement);
    }
    
    /**
     *Run a pre-loaded script by calling the named Sub or Function with the supplied parameters.
     *If a Function was called the return value of that Function is returned.
     */
    public String Run(String startsub, String[] subparams) {
    	return MSmethods.Run(startsub, subparams);
    }
    
    /**
     *Returns the original type of the result as the actual result is converted to a string value by Eval and Run.
     */
    public String ResultType() {
    	return MSmethods.ResultType();    	
    }
   
    /**
     *The ClearError method is used to reset the Error object.
     *Using the AddCode, Eval, ExecuteStatement, or Reset methods will also clear the Error object before these methods begin processing. 
     */
    public void ClearError() {
    	MSmethods.ClearError();
    }
    
    /**
     *Returns True if an error occurred during script execution. The other ErrorXxxx methods give more information
     */
    public boolean Errored() {
    	return MSmethods.Errored();
    }
    
    /**
     * Returns the source code column number where the error occurred.
     */
    public int ErrorColumn() {
    	return MSmethods.ErrorColumn();
    }
    
    /**
     *Returns a description of the error.
     */
    public String ErrorDescription() {
    	return MSmethods.ErrorDescription();   	
    }
    
    /**
     *Returns the source code line number where the error occurred
     */
    public int ErrorLine() {
    	return MSmethods.ErrorLine();
    }
    
    /**
     *Returns the error number.
     */
    public int ErrorNumber() {
    	return MSmethods.ErrorNumber();
    }
    
    /**
     *Returns a description of the general type of error. 
     */
    public String ErrorSource() {
    	return MSmethods.ErrorSource();
    }
    
    /**
     *Returns the line of source code where the error occurred. 
     */
    public String ErrorText() {
    	return MSmethods.ErrorText();
    }
    
    /**
     *Returns True if  the script timed out.
     */
    public boolean TimedOut() {
    	return MSmethods.TimedOut();    	
    }
	
	
    /**
     *Returns True if an Exception occurred running a script or evaluating an expression.
     */
    public boolean ExceptionFlag() {
    	return MSmethods.ExceptionFlag();
    }
    
    /**
     *Returns the message of whatever Exception occurred when running a script or evaluating an expression.
     */
    public String ExceptionMsg() {
    	return MSmethods.ExceptionMsg();
    	
    }

}
	
	